@@ -5,7 +5,7 @@
  | program/include/rcube_message.php                                     |
  |                                                                       |
  | This file is part of the Roundcube Webmail client                     |
- | Copyright (C) 2008-2010, Roundcube Dev. - Switzerland                 |
+ | Copyright (C) 2008-2010, The Roundcube Dev Team                       |
  | Licensed under the GNU GPL                                            |
  |                                                                       |
  | PURPOSE:                                                              |
@@ -15,7 +15,7 @@
  | Author: Thomas Bruederli <roundcube@gmail.com>                        |
  +-----------------------------------------------------------------------+
 
- $Id: rcube_message.php 4643 2011-04-11 12:24:00Z alec $
+ $Id: rcube_message.php 5514 2011-11-30 11:35:43Z alec $
 
 */
 
@@ -48,7 +48,6 @@ class rcube_message
 
     public $uid = null;
     public $headers;
-    public $structure;
     public $parts = array();
     public $mime_parts = array();
     public $attachments = array();
@@ -77,7 +76,7 @@ class rcube_message
         $this->imap->get_all_headers = true;
 
         $this->uid = $uid;
-        $this->headers = $this->imap->get_headers($uid, NULL, true, true);
+        $this->headers = $this->imap->get_message($uid);
 
         if (!$this->headers)
             return;
@@ -94,9 +93,9 @@ class rcube_message
                 '_mbox' => $this->imap->get_mailbox_name(), '_uid' => $uid))
         );
 
-        if ($this->structure = $this->imap->get_structure($uid, $this->headers->body_structure)) {
-            $this->get_mime_numbers($this->structure);
-            $this->parse_structure($this->structure);
+        if (!empty($this->headers->structure)) {
+            $this->get_mime_numbers($this->headers->structure);
+            $this->parse_structure($this->headers->structure);
         }
         else {
             $this->body = $this->imap->get_body($uid);
@@ -143,10 +142,10 @@ class rcube_message
      * @param string $mime_id Part MIME-ID
      * @return string URL or false if part does not exist
      */
-    public function get_part_url($mime_id)
+    public function get_part_url($mime_id, $embed = false)
     {
         if ($this->mime_parts[$mime_id])
-            return $this->opt['get_url'] . '&_part=' . $mime_id;
+            return $this->opt['get_url'] . '&_part=' . $mime_id . ($embed ? '&_embed=1' : '');
         else
             return false;
     }
@@ -286,7 +285,7 @@ class rcube_message
         if ($message_ctype_primary == 'text' && !$recursive) {
             $structure->type = 'content';
             $this->parts[] = &$structure;
-            
+
             // Parse simple (plain text) message body
             if ($message_ctype_secondary == 'plain')
                 foreach ((array)$this->uu_decode($structure) as $uupart) {
@@ -299,14 +298,16 @@ class rcube_message
             $structure->type = 'content';
             $this->parts[] = &$structure;
         }
-        // message contains alternative parts
-        else if ($mimetype == 'multipart/alternative' && is_array($structure->parts)) {
+        // message contains (more than one!) alternative parts
+        else if ($mimetype == 'multipart/alternative'
+            && is_array($structure->parts) && count($structure->parts) > 1
+        ) {
             // get html/plaintext parts
             $plain_part = $html_part = $print_part = $related_part = null;
 
             foreach ($structure->parts as $p => $sub_part) {
                 $sub_mimetype = $sub_part->mimetype;
-        
+
                 // check if sub part is
                 if ($sub_mimetype == 'text/plain')
                     $plain_part = $p;
@@ -323,7 +324,7 @@ class rcube_message
                 $this->parse_alternative = true;
                 $this->parse_structure($structure->parts[$related_part], true);
                 $this->parse_alternative = false;
-        
+
                 // if plain part was found, we should unset it if html is preferred
                 if ($this->opt['prefer_html'] && count($this->parts))
                     $plain_part = null;
@@ -373,6 +374,8 @@ class rcube_message
             $p->ctype_secondary = 'plain';
             $p->body            = rcube_label('encryptedmessage');
             $p->size            = strlen($p->body);
+
+            $this->parts[] = $p;
         }
         // message contains multiple parts
         else if (is_array($structure->parts) && !empty($structure->parts)) {
@@ -432,7 +435,7 @@ class rcube_message
                         $this->attachments[] = $mail_part;
                 }
                 // part message/*
-                else if ($primary_type=='message') {
+                else if ($primary_type == 'message') {
                     $this->parse_structure($mail_part, true);
 
                     // list as attachment as well (mostly .eml)
@@ -496,36 +499,41 @@ class rcube_message
                         $this->attachments[] = $mail_part;
                     }
                 }
+                // attachment part as message/rfc822 (#1488026)
+                else if ($mail_part->mimetype == 'message/rfc822') {
+                    $this->parse_structure($mail_part);
+                }
             }
 
             // if this was a related part try to resolve references
             if ($mimetype == 'multipart/related' && sizeof($this->inline_parts)) {
                 $a_replaces = array();
+                $img_regexp = '/^image\/(gif|jpe?g|png|tiff|bmp|svg)/';
 
                 foreach ($this->inline_parts as $inline_object) {
-                    $part_url = $this->get_part_url($inline_object->mime_id);
+                    $part_url = $this->get_part_url($inline_object->mime_id, true);
                     if ($inline_object->content_id)
                         $a_replaces['cid:'.$inline_object->content_id] = $part_url;
                     if ($inline_object->content_location) {
                         $a_replaces[$inline_object->content_location] = $part_url;
                     }
-                    // MS Outlook sends sometimes non-related attachments as related
-                    // In this case multipart/related message has only one text part
-                    // We'll add all such attachments to the attachments list
-                    if (!isset($got_html_part) && empty($inline_object->content_id)
-                        && !empty($inline_object->filename)
-                    ) {
-                        $this->attachments[] = $inline_object;
-                    }
-                    // MS Outlook sometimes also adds non-image attachments as related
-                    // We'll add all such attachments to the attachments list
-                    // Warning: some browsers support pdf in <img/>
-                    // @TODO: we should fetch HTML body and find attachment's content-id
-                    // to handle also image attachments without reference in the body
-                    if (!empty($inline_object->filename)
-                        && !preg_match('/^image\/(gif|jpe?g|png|tiff|bmp|svg)/', $inline_object->mimetype)
-                    ) {
-                        $this->attachments[] = $inline_object;
+
+                    if (!empty($inline_object->filename)) {
+                        // MS Outlook sends sometimes non-related attachments as related
+                        // In this case multipart/related message has only one text part
+                        // We'll add all such attachments to the attachments list
+                        if (!isset($got_html_part) && empty($inline_object->content_id)) {
+                            $this->attachments[] = $inline_object;
+                        }
+                        // MS Outlook sometimes also adds non-image attachments as related
+                        // We'll add all such attachments to the attachments list
+                        // Warning: some browsers support pdf in <img/>
+                        else if (!preg_match($img_regexp, $inline_object->mimetype)) {
+                            $this->attachments[] = $inline_object;
+                        }
+                        // @TODO: we should fetch HTML body and find attachment's content-id
+                        // to handle also image attachments without reference in the body
+                        // @TODO: should we list all image attachments in text mode?
                     }
                 }
 
